from ..faultsbase import FaultsBase

{% for header in headers %}
{% for c in header.classes %}

class {{ c.name }}(FaultsBase):
    __slots__ = ()

    {% for prop in c.properties.public %}
    @property
    def {% if prop.name == "APIError" %}apiError{% else %}{{ prop.name[0].lower() + prop.name[1:] }}{% endif %}(self) -> bool:
        return self.bits & (1 << {{ loop.index0 }}) != 0

    {% endfor %}
{% endfor %}
{% endfor %}
