/**
    This file is autogenerated
*/

{
  py::class_<BuffTrajPointStream> cls(m, "BuffTrajPointStream");
  cls.def(py::init<>())
    .def("_create1", &BuffTrajPointStream::_create1, release_gil())
    .def("clear", &BuffTrajPointStream::clear, release_gil())
    .def("_write", &BuffTrajPointStream::_write, py::arg("position"), py::arg("velocity"), py::arg("arbFeedFwd"), py::arg("auxiliaryPos"), py::arg("auxiliaryVel"), py::arg("auxiliaryArbFeedFwd"), py::arg("profileSlotSelect0"), py::arg("profileSlotSelect1"), py::arg("isLastPoint"), py::arg("zeroPos"), py::arg("timeDur"), py::arg("useAuxPID"), release_gil())
  ;
}